<?php
/*
	Front Controller
	Created: December 26, 2012
	Updated: 
	Troy Boileau

	This is meant to be used with my full development framework.

	Examine this directory structure:
	/
		www/
			templates/
				frame.tpl.php
				views/
					example_view.tpl.php
					index.tpl.php
					example_module/
						example_module_view.tpl.php
						_sidebar.tpl.php
					example_application/
						_header.tpl.php
						example_application_module/
							example_application_module_view.tpl.php

	www.foo.com
		will display index.tpl.php
	www.foo.com/example_view
		will display example_view.tpl.php
	www.foo.com/example_module/example_module_view
		will display example_module_view.tpl.php
	www.foo.com/example_application/example_application_module/example_application_module_view
		will display example_application_module_view.tpl.php

	For internal views (application/module specific sidebars etc.) you can use:
		$front_controller = new FrontController();
		$view = new Savant3();
		$view->sidebar = $view->fetch($front_controller->getModuleString() . '_sidebar.tpl.php');

	NOTE that any file or folder starting with an underscore (_) is not accessible through the URL.

*/

class FrontController {
	private $path;
	private $application;
	private $module;
	private $view;

	public function __construct() {
		global $config;
		$path = $this->getPath();
		if ($path == false) {
			$this->view = $config->default_view;
		} else {
			$count_path = count($path);
			switch ($count_path) {
				case 1:
					$view = $path[0];
					if (strpos($view, "_") === 0 ||
						$view . '.tpl.php' == $config->default_view ||
						!file_exists($_SERVER['DOCUMENT_ROOT'] . $config->views_folder . $view . '.tpl.php'))
						$this->redirect();
					$this->view = $view;
					break;
				case 2:
					$module = $path[0];
					$view = $path[1];
					if (strpos($view, "_") === 0 ||
						strpos($module, "_") === 0 ||
						!file_exists($_SERVER['DOCUMENT_ROOT'] . $config->views_folder . $module . '/' . $view . '.tpl.php'))
						$this->redirect();
					$this->view = $view;
					$this->module = $module;
					break;
				case 3:
					$application = $path[0];
					$module = $path[1];
					$view = $path[2];
					if (strpos($view, "_") === 0 ||
						strpos($module, "_") === 0 ||
						strpos($application, "_") === 0 ||
						!file_exists($_SERVER['DOCUMENT_ROOT'] . $config->views_folder . $application . '/' . $module . '/' . $view . '.tpl.php'))
						$this->redirect();
					$this->view = $view;
					$this->module = $module;
					$this->application = $application;
					break;
				default:
					$this->view = $config->default_view;
					break;
			}
		}
	}

	private function buildPath() {
		if (empty($path)) {
			$url = parse_url("http://".$_SERVER["HTTP_HOST"].$_SERVER["REQUEST_URI"]);
			if ($url['path'] !== '/') {
				$path = explode('/', $url['path']);
				array_shift($path);
			}
			if (!empty($path))
				$this->path = $path;
		}
	}
	public function redirect($location = '') {
		global $config;
		if ($location == '')
			$location = $config->home_url;
		header('Location: ' . $location);
	}
	public function getPath() {
		if (empty($this->path))
			$this->buildPath();
		if (!empty($this->path))
			return $this->path;
		return false;
	}
	public function getApplication() {
		if (!empty($this->application))
			return $this->application;
		return;
	}
	public function getModule() {
		if (!empty($this->module))
			return $this->module;
		return;
	}
	public function getView() {
		if (!empty($this->view))
			return $this->view;
		return;
	}
	public function getViewString() {
		global $config;
		$arr = array();
		if (!empty($this->application)) $arr[] = $this->application;
		if (!empty($this->module)) $arr[] = $this->module;
		if (!empty($this->view)) $arr[] = $this->view;

		return $config->views_folder . implode('/', $arr);
	}
	public function getModuleString() {
		global $config;
		$arr = array();
		if (!empty($this->application)) $arr[] = $this->application;
		if (!empty($this->module)) $arr[] = $this->module;

		return  $config->views_folder . implode('/', $arr);
	}
	public function getApplicationString() {
		global $config;
		$arr = array();
		if (!empty($this->application)) $arr[] = $this->application;

		return  $config->views_folder . implode('/', $arr);
	}
}


?>